.\" Automatically generated from an mdoc input file.  Do not edit.
.\" Copyright (c) 2024 Andrea Biscuola <a@abiscuola.com>
.\"
.\" Permission to use, copy, modify, and distribute this software for any
.\" purpose with or without fee is hereby granted, provided that the above
.\" copyright notice and this permission notice appear in all copies.
.\"
.\" THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
.\" WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
.\" MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
.\" ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
.\" WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
.\" ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
.\" OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
.\"
.TH "IRCTK-EXT-SDK" "n" "May 20, 2024" "OpenBSD 7.5" "n"
.nh
.if n .ad l
.SH "NAME"
\fBirctk-ext-sdk\fR
\- tcl software development kit for irctk extensions
.SH "SYNOPSIS"
\fB::irctk::filter\fR \fItype\fR [\fIcallback\fR]
.PP
\fB::irctk::init\fR \fIname\fR \fIversion\fR \fIcapabilities\fR [\fIcallback\fR]
.PP
\fB::irctk::ircsend\fR \fInet\fR \fIchan\fR \fIcmd\fR \fIline\fR
.PP
\fB::irctk::loop\fR
.SH "DESCRIPTION"
The IRCTk extensions SDK, is a tcl package designed to simplify the creation
of IRCTk extensions in TCL.
.PP
The subsystem calls any
\fIcallback\fR
defined in the extension with a message as an argument to it.
A call to a callback have the following form:
.nf
.sp
.RS 6n
callback msg
.RE
.fi
.PP
Messages passed to a callback are tcl dictionaries with different fields based
on the type.
The
\(lqirc\(rq
message type has the following fields:
.TP 8n
id
Message Id.
.TP 8n
type
Message type.
.TP 8n
timestamp
Timestamp of the message, in seconds, from the epoch.
.TP 8n
cid
The id of the channel in IRCTk.
.TP 8n
nick
The nick of the user that sent the message
.TP 8n
level
Message level, from
\(lqinfo\(rq,
to
\(lqmention.\(rq
.TP 8n
focus
true or false.
Define if the application window is on focus.
.TP 8n
status
Client user's status.
Might be
\(lqaway\(rq
or not.
.TP 8n
network
.br
The network name.
.TP 8n
channel
.br
The channel name.
.TP 8n
tags
A list of IRCv3 tags
.TP 8n
command
.br
The IRC command.
.TP 8n
args
The command arguments.
.PP
Fields for the
\(lqplumb\(rq
message are:
.TP 8n
id
Message Id (EMPTY).
.TP 8n
type
The message type.
.TP 8n
cid
The Id of the channel where the plumb happened.
.TP 8n
network
.br
The network name.
.TP 8n
channel
.br
The channel name.
.TP 8n
data
The text string.
.PP
\fB::irctk::init\fR
initializes the subsystem and perform the initial handshake with IRCTk using
\fIname\fR
and
\fIversion\fR
as the extension's parameters.
The
\fIcapabilities\fR
argument is a list of IRCv3 capabilities the extension is able to manage, that
IRCTk will ask the server to enable.
The optional
\fIcallback\fR,
if provided, will handle all the messages routed to the extension without a
proper filter in place.
.PP
\fB::irctk::filter\fR
defines a new policy for IRCTk to follow, related to the extension.
In particular, it informs IRCTk to route only specific message types,
or specific message commands, to the extension itself.
The optional
\fIcallback\fR
will be called, by the sdk, whenever a message of the specified
\fItype\fR
is received.
The possible message types are:
.TP 8n
\fB\(bu\fR
plumb
.TP 8n
\fB\(bu\fR
irc
.PP
As for the type irc, it's also possible to specify filters for specific
commands, by first issuing a filter request for the irc type, and
then by issuing a request for the commands themselves:
.nf
.sp
.RS 6n
::irctk::filter irc
::irctk::filter JOIN "callback"
::irctk::filter PRIVMSG "callback"
.RE
.fi
.PP
Issuing a request only for the irc type, like the following:
.nf
.sp
.RS 6n
::irctk::filter irc "callback"
.RE
.fi
.PP
Will cause IRCTk to route all the messages of type irc to the extension.
.PP
The
\fB::irc::loop\fR
command starts the main extension loop.
In essence, it starts the tcl event loop, processing incoming messages, and
routing them to the appropriate callback for the required filters, or using the
default handler if the message type doesn't have a filter specified.
.PP
It's possible, of course, for an extension to add it's own events to the event
loop, as the sdk just takes care of starting the standard tcl one.
An example may be an extension that open a socket, binding a read event
to get some data from it.
.PP
The
\fB::irctk::ircsend\fR
command, will send a message of type IRC to IRCTk, with
\fInet\fR
as a network of destination,
\fIchan\fR
as the specific channel,
\fIcmd\fR
as the IRC command and
\fIline\fR
as a string for the given command.
.PP
Sending an irc message to a channel that doesn't exists in the specified
network, will cause IRCTk to create a, so called,
\(lqread only\(rq
channel, where the body of the message will be visualized, but without any
possibility for the user to interact with the channel itself.
.PP
This is useful, for example, to create extensions that doesn't need to send
messages to an IRC server, but only show information to the client's user.
.SH "ENVIRONMENT"
.TP 8n
\fRIRCTK_TCL_SDK\fR
Path to the SDk source file.
.TP 8n
\fRIRCTK_LOGS_PATH\fR
Path to the IRCTk chat logs directory.
.SH "EXAMPLES"
The following is a, basic,
\(lqHello, world!\(rq
Extension, showing the general layout:
.nf
.sp
.RS 6n
#!/usr/bin/env tclsh8.6

source $::env(IRCTK_TCL_SDK)

set name "Hello"
set version 1.0.0

proc helloworld {msg} {
	set network [dict get $msg network]
	set channel [dict get $msg channel]

	::irctk::ircsend $network $channel PRIVMSG "Hello world!"
}

::irctk::init $name $version {}
::irctk::filter irc {}
::irctk::filter hello helloworld

::irctk::loop
.RE
.fi
.SH "SEE ALSO"
irctk-extensions(7)
.SH "AUTHORS"
Andrea Biscuola <\fIa@abiscuola.com\fR>
