/*
 * Copyright (c) 2024 Andrea Biscuola <a@abiscuola.com>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#ifndef _FCHAT_H_
#define _FCHAT_H_

#define FCHATMAXURLSIZE		2084
#define FCHATMAXPASSSIZE		256
#define FCHATMIMEMAX		256
#define FCHATMUSRS		64

struct fchat {
	struct	api *capi;
	char	url[FCHATMAXURLSIZE];
	char	pass[FCHATMAXPASSSIZE];
	char	user[FCHATMUSRS];
	uint64_t	cid;
};

STAILQ_HEAD(fchatmsgs, fchatmsg);
struct fchatmsg {
	char			fname[PATH_MAX];
	char			fmime[FCHATMIMEMAX];
	char			user[FCHATMUSRS];
	char			*text;
	uint64_t			id;
	uint64_t			mdel;
	time_t			ts;
	time_t			lms;
	size_t			fsize;
	int			error;
	STAILQ_ENTRY(fchatmsg)	next;
};

int		 fchatisdel(const struct fchatmsg *);
int		 fchaterror(struct fchatmsg *);
char		*fchatfiledownload(struct fchat *, struct fchatmsg *);
char		*fchatfilemime(struct fchatmsg *);
char		*fchatfilename(struct fchatmsg *);
size_t		 fchatfilesize(struct fchatmsg *);
void		 fchatfree(struct fchat *);
uint64_t		 fchatid(struct fchatmsg *);
int		 fchatinit(void);
time_t		 fchatlocaltime(struct fchatmsg *);
void		 fchatmsgsfree(struct fchatmsgs *);
struct fchat	*fchatnew(const char *, const char *, const char *);
int		 fchatnext(struct fchatmsgs *, struct fchatmsg **);
struct fchatmsgs	*fchatpoll(struct fchat *, int64_t);
int		 fchatsend(struct fchat *, const char *, char *);
char		*fchattext(struct fchatmsg *);
time_t		 fchattime(struct fchatmsg *);
char		*fchatuser(struct fchatmsg *);

#endif /* _FCHAT_H_ */
