/*
 * Copyright (c) 2024 Andrea Biscuola <a@abiscuola.com>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#ifndef _IRC_H_
#define _IRC_H_

#define IRCMSGLEN		8703
#define IRCTXTLEN		4096
#define IRCTOPICLEN	512
#define IRCSRCLEN		256
#define IRCHOSTLEN		256
#define IRCTAGVALLEN	256
#define IRCTAGKEYLEN	128
#define IRCCHANLEN	32
#define IRCCMDLEN		32

enum irccode {
	IRCRPLWELCOME = 001,
	IRCRPLYOURHOST = 002,
	IRCRPLCREATED = 003,
	IRCRPLMYINFO = 004,
	IRCRPLISUPPORT = 005,
	IRCRPLBOUNCE = 010,
	IRCRPLSTATSCOMMANDS = 212,
	IRCRPLENDOFSTATS = 219,
	IRCRPLUMODEIS = 221,
	IRCRPLSTATSUPTIME = 242,
	IRCRPLLUSERCLIENT = 251,
	IRCRPLLUSEROP = 252,
	IRCRPLLUSERUNKNOWN = 253,
	IRCRPLLUSERCHANNELS = 254,
	IRCRPLLUSERME = 255,
	IRCRPLADMINME = 256,
	IRCRPLADMINLOC1 = 257,
	IRCRPLADMINLOC2 = 258,
	IRCRPLADMINEMAIL = 259,
	IRCRPLTRYAGAIN = 263,
	IRCRPLLOCALUSERS = 265,
	IRCRPLGLOBALUSERS = 266,
	IRCRPLWHOISCERTFP = 276,
	IRCRPLNONE = 300,
	IRCRPLAWAY = 301,
	IRCRPLUSERHOST = 302,
	IRCRPLUNAWAY = 305,
	IRCRPLNOWAWAY = 306,
	IRCRPLWHOISREGNICK = 307,
	IRCRPLWHOISUSER = 311,
	IRCRPLWHOISSERVER = 312,
	IRCRPLWHOISOPERATOR = 313,
	IRCRPLWHOWASUSER = 314,
	IRCRPLENDOFWHO = 315,
	IRCRPLWHOISIDLE = 317,
	IRCRPLENDOFWHOIS = 318,
	IRCRPLWHOISCHANNELS = 319,
	IRCRPLWHOISSPECIAL = 320,
	IRCRPLLISTSTART = 321,
	IRCRPLLIST = 322,
	IRCRPLLISTEND = 323,
	IRCRPLCHANNELMODEIS = 324,
	IRCRPLCREATIONTIME = 329,
	IRCRPLWHOISACCOUNT = 330,
	IRCRPLNOTOPIC = 331,
	IRCRPLTOPIC = 332,
	IRCRPLTOPICWHOTIME = 333,
	IRCRPLINVITELIST = 336,
	IRCRPLENDOFINVITELIST = 337,
	IRCRPLWHOISACTUALLY = 338,
	IRCRPLINVITING = 341,
	IRCRPLINVEXLIST = 346,
	IRCRPLENDOFINVEXLIST = 347,
	IRCRPLEXCEPTLIST = 348,
	IRCRPLENDOFEXCEPTLIST = 349,
	IRCRPLVERSION = 351,
	IRCRPLWHOREPLY = 352,
	IRCRPLNAMREPLY = 353,
	IRCRPLLINKS = 364,
	IRCRPLENDOFLINKS = 365,
	IRCRPLENDOFNAMES = 366,
	IRCRPLBANLIST = 367,
	IRCRPLENDOFBANLIST = 368,
	IRCRPLENDOFWHOWAS = 369,
	IRCRPLINFO = 371,
	IRCRPLMOTD = 372,
	IRCRPLENDOFINFO = 374,
	IRCRPLMOTDSTART = 375,
	IRCRPLENDOFMOTD = 376,
	IRCRPLWHOISHOST = 378,
	IRCRPLWHOISMODES = 379,
	IRCRPLYOUREOPER = 381,
	IRCRPLREHASHING = 382,
	IRCRPLTIME = 391,
	IRCERRUNKNOWNERROR = 400,
	IRCERRNOSUCHNICK = 401,
	IRCERRNOSUCHSERVER = 402,
	IRCERRNOSUCHCHANNEL = 403,
	IRCERRCANNOTSENDTOCHAN = 404,
	IRCERRTOOMANYCHANNELS = 405,
	IRCERRWASNOSUCHNICK = 406,
	IRCERRNOORIGIN = 409,
	IRCERRNORECIPIENT = 411,
	IRCERRNOTEXTTOSEND = 412,
	IRCERRINPUTTOOLONG = 417,
	IRCERRUNKNOWNCOMMAND = 421,
	IRCERRNOMOTD = 422,
	IRCERRNONICKNAMEGIVEN = 431,
	IRCERRERRONEUSNICKNAME = 432,
	IRCERRNICKNAMEINUSE = 433,
	IRCERRNICKCOLLISION = 436,
	IRCERRUSERNOTINCHANNEL = 441,
	IRCERRNOTONCHANNEL = 442,
	IRCERRUSERONCHANNEL = 443,
	IRCERRNOTREGISTERED = 451,
	IRCERRNEEDMOREPARAMS = 461,
	IRCERRALREADYREGISTERED = 462,
	IRCERRPASSWDMISMATCH = 464,
	IRCERRYOUREBANNEDCREEP = 465,
	IRCERRCHANNELISFULL = 471,
	IRCERRUNKNOWNMODE = 472,
	IRCERRINVITEONLYCHAN = 473,
	IRCERRBANNEDFROMCHAN = 474,
	IRCERRBADCHANNELKEY = 475,
	IRCERRBADCHANMASK = 476,
	IRCERRNOPRIVILEGES = 481,
	IRCERRCHANOPRIVSNEEDED = 482,
	IRCERRCANTKILLSERVER = 483,
	IRCERRNOOPERHOST = 491,
	IRCERRUMODEUNKNOWNFLAG = 501,
	IRCERRUSERSDONTMATCH = 502,
	IRCERRHELPNOTFOUND = 524,
	IRCERRINVALIDKEY = 525,
	IRCRPLSTARTTLS = 670,
	IRCRPLWHOISSECURE = 671,
	IRCERRSTARTTLS = 691,
	IRCERRINVALIDMODEPARAM = 696,
	IRCRPLHELPSTART = 704,
	IRCRPLHELPTXT = 705,
	IRCRPLENDOFHELP = 706,
	IRCERRNOPRIVS = 723,
	IRCRPLLOGGEDIN = 900,
	IRCRPLLOGGEDOUT = 901,
	IRCERRNICKLOCKED = 902,
	IRCRPLSASLSUCCESS = 903,
	IRCERRSASLFAIL = 904,
	IRCERRSASLTOOLONG = 905,
	IRCERRSASLABORTED = 906,
	IRCERRSASLALREADY = 907,
	IRCRPLSASLMECHS = 908
};

struct ircsrv {
	char	host[IRCHOSTLEN];
	char	*l;
	FILE	*in;
	FILE	*out;
	size_t	bs;
};

STAILQ_HEAD(ircmsgtags, ircmsgtag);
struct ircmsgtag {
	char			val[IRCTAGVALLEN];
	char			key[IRCTAGKEYLEN];
	STAILQ_ENTRY(ircmsgtag)	next;
};

struct ircmsg {
	char	src[IRCSRCLEN];
	char	cmd[IRCCMDLEN];
	char	text[IRCTXTLEN];
	struct	ircmsgtags tags;
};

struct ircsrv	*ircnew(FILE *, FILE *);
void		 ircfree(struct ircsrv *);
int		 ircget(struct ircsrv *, struct ircmsg **);
int		 ircmsgcode(struct ircsrv *, enum irccode, const char *, const char *, ...);
int		 ircmsgjoin(struct ircsrv *, const char *, const char *);
int		 ircmsgping(struct ircsrv *, const char *, const char *);
int		 ircmsgpong(struct ircsrv *, const char *, const char *);
int		 ircmsgpriv(struct ircsrv *, const char *, const char *, const char *);
int		 ircmsgset(struct ircmsg **, const char *, const char *, const char *, ...);
char		*ircmsgcmd(struct ircmsg *);
int		 ircmsgaddtag(struct ircmsg *, const char *, const char *);
void		 ircmsgfree(struct ircmsg *);
int		 ircmsgsend(struct ircsrv *, struct ircmsg *);
char		*ircmsgsrc(struct ircmsg *);
char		*ircmsgtext(struct ircmsg *);

#endif /* _IRC_H_ */
