/* $Id: api.h,v 1.17 2021/03/11 10:12:01 absc Exp $ */
/*
 * Copyright (c) 2018-2020 Andrea Biscuola <a@abiscuola.com>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#ifndef _API_H_
#define _API_H_

#define APIURLLEN	512

typedef enum {
	APIREQGET,
	APIREQPOST,
	APIREQPUT,
	APIREQDEL,
	APIREQERR
} reqtype;

struct api {
	struct		curl_slist *hdr;
	curl_mime	*mime;
	char		url[APIURLLEN];
	CURL		*hnd;
	char		*b;
	char		*np;
	char		cf[PATH_MAX];
	size_t		s;
	size_t		bs;
	void		*tk;
	void		*tp;
	size_t		ts;
	int		cp;
	int		tc;
};

typedef enum {
	APIJSONVAL,
	APIJSONOBJ,
	APIJSONARR,
	APIJSONSTR
} apijsont;

typedef enum {
	APIFALSE,
	APITRUE
} apibool;

struct apiv {
	union {
		char	*val;
		size_t	size;
	};
	apijsont	type;
};

int		 apiglobalinit(char **);
struct api		*apiinit(const char *, const char *);
void		 apifree(struct api *);
int		 apiaddformfield(struct api *, const char *,
		     const char *, const char *, size_t);
int		 apiaddstr(struct api *, const char *, const char *);
int		 apiaddint(struct api *, const char *, const long long);
int		 apiaddfloat(struct api *, const char *, const double);
int		 apiaddbool(struct api *, const char *, const apibool);
int		 apiaddobj(struct api *, const char *);
int		 apicloseobj(struct api *);
int		 apiaddarray(struct api *, const char *);
int		 apiclosearray(struct api *);
int		 apiaddnull(struct api *, const char *);
int		 apiclose(struct api *);
int		 apiaddheader(struct api *, const char *, const char *);
int		 apisetauth(struct api *, const char *, const char *);
int		 apisetbearer(struct api *, const char *);
int		 apisetcapath(struct api *, const char *);
int		 apisetcookiefile(struct api *, const char *);
char		*apicookiefile(struct api *);
int		 apido(struct api *);
char		*apierror(struct api *);
int		 apireset(struct api *, const char *, const char *);
long		 apihttpcode(struct api *);
ssize_t		 apidump(struct api *, char **);
int		 apijsonnext(struct api *, struct apiv *);

#endif /* !_API_H_ */
