/*	$Id: t-headers-multi.c,v 1.2 2020/08/06 21:24:46 absc Exp $	*/
/*
 * Copyright (c) 2020 Andrea Biscuola <a@abiscuola.com>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <err.h>
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <curl/curl.h>
#include <api.h>

int
main(int argc, char *argv[])
{
	struct apiv	v;
	struct api		*rpc;
	char		*e;

	if (apiglobalinit(&e) == -1)
		errx(1, "%s", e);

	rpc = apiinit("GET", "https://postman-echo.com/headers");
	if (!rpc)
		errx(1, "%s", "initialization failed");

	if (apiaddheader(rpc, "t-headers-simple", "t-headers-simple") == -1)
		errx(1, "%s", "header adding failed");
	if (apiaddheader(rpc, "X-Custom-Auth", "user") == -1)
		errx(1, "%s", "header adding failed");
	if (apiaddheader(rpc, "X-Custom-Basic", "password") == -1)
		errx(1, "%s", "header adding failed");

	if (apido(rpc) == -1)
		errx(1, "%ld: %s", apihttpcode(rpc), apierror(rpc));

	while (apijsonnext(rpc, &v) != -1) {
		if (v.type != APIJSONSTR)
			continue;

		if (strcmp(v.val, "t-headers-simple") == 0) {
			if (apijsonnext(rpc, &v) == -1)
				return 1;
			printf("%s:%s\n", "t-headers-simple", v.val);
		} else if (strcasecmp(v.val, "X-Custom-Auth") == 0) {
			if (apijsonnext(rpc, &v) == -1)
				return 1;
			printf("%s:%s\n", "X-Custom-Auth", v.val);
		} else if (strcasecmp(v.val, "X-Custom-Basic") == 0) {
			if (apijsonnext(rpc, &v) == -1)
				return 1;
			printf("%s:%s", "X-Custom-Basic", v.val);
		}
	}

	apifree(rpc);

	return 0;
}
